using System;
using System.Collections.Generic;
using ReactiveUI;
using Captchapp.Models;
using Avalonia.Controls.Shapes;
using Avalonia.Controls;
using Avalonia.Media;


namespace Captchapp.ViewModels
{
    public class Page1ViewModel : ReactiveObject
    {
        private string email = "";
        private string password = "";
        private string errorMessage = "   ";
        private bool errorVisibility = false;
        private string timerMessage = "   ";
        private bool timerVisibility = false;
        private int canvaWidth = 300;
        private int canvaHeight = 150;
        private string captchaText = "";
        private string captchaEntry;
        private bool captchaVisibility = false;
        private bool loginVisibility = true;
        private bool textBoxEnable = true;
        private Canvas canva;

        //    
        public bool CaptchaVisibility 
        {
            get => captchaVisibility;
            set => this.RaiseAndSetIfChanged(ref captchaVisibility, value);
        }

        //      
        public bool TextBoxEnable
        {
            get => textBoxEnable;
            set => this.RaiseAndSetIfChanged(ref textBoxEnable, value);
        }

        //   
        public bool LoginVisibility
        {
            get => loginVisibility;
            set => this.RaiseAndSetIfChanged(ref loginVisibility, value);
        }

        //   email
        public string Email
        {
            get => email;
            set => this.RaiseAndSetIfChanged(ref email, value);
        }

        //   
        public string CaptchaEntry 
        {
            get => captchaEntry;
            set => this.RaiseAndSetIfChanged(ref captchaEntry, value);
        }

        //  
        public string CaptchaText
        {
            get => captchaText;
            set => this.RaiseAndSetIfChanged(ref captchaText, value);
        }

        //   
        public string Password
        {
            get => password;
            set => this.RaiseAndSetIfChanged(ref password, value);
        }

        //     
        private string ModelPassword {
            get => UserModel.password;
        }

        //     
        private string ModelEmail {
            get => UserModel.login;
        }

        //  
        public bool ErrorVisibility {
            get => errorVisibility;
            set => this.RaiseAndSetIfChanged(ref errorVisibility, value);
        }

        //  
        public bool TimerVisibility
        {
            get => timerVisibility;
            set => this.RaiseAndSetIfChanged(ref timerVisibility, value);
        }

        //   
        public string ErrorMessage
        {
            get => errorMessage;
            set => errorMessage = value;
        }

        //      
        public string TimerMessage
        {
            get => timerMessage;
            set => this.RaiseAndSetIfChanged(ref timerMessage, value);
        }

        // 
        public Canvas Canva
        {
            get => canva;
            set => this.RaiseAndSetIfChanged(ref canva, value);
        }

        public bool Authorize()
        {
            if (ModelPassword == Password && ModelEmail == Email)
                return true;
            else
                return false;
        }

        public void makeCaptcha() {
            Random rnd = new Random();
            SolidColorBrush color = new SolidColorBrush(Color.FromArgb(Convert.ToByte(rnd.Next(70)), Convert.ToByte(rnd.Next(256)), Convert.ToByte(rnd.Next(255)), Convert.ToByte(rnd.Next(255)))); // Color.FromRgb(Convert.ToByte(rnd.Next(256)), Convert.ToByte(rnd.Next(255)), Convert.ToByte(rnd.Next(255)))
            
            Canvas canvas = new Canvas()
            {
                Width = canvaWidth,
                Height = canvaHeight,
                Background = color
            };

            int left = -15;
            int numCount = rnd.Next(7, 11); 
            string chars = "";

            for (int i = 0; i < numCount; i++)
            {
                chars += (char)(new int[] { rnd.Next(48, 58), rnd.Next(97, 123) }[rnd.Next(2)]);
                TextBlock letter = new TextBlock()
                {
                    Text = chars[i].ToString(),
                    FontSize = 30, //rnd.Next(30, 50)
                    Foreground = Brushes.Black,
                    RenderTransform = new RotateTransform(new int[] { rnd.Next(285, 361), rnd.Next(76) }[rnd.Next(2)]),
                    Margin = new Avalonia.Thickness(10),
                    FontWeight = new FontWeight[] { FontWeight.Bold, FontWeight.Regular }[rnd.Next(2)],
                    FontStyle = new FontStyle[] { FontStyle.Italic, FontStyle.Normal }[rnd.Next(2)]
                };

                Canvas.SetLeft(letter, rnd.Next(left + 15, left + ((canvaWidth - 100) / numCount)));
                left += (canvaWidth - 100) / numCount;
                Canvas.SetTop(letter, rnd.Next(100));
                canvas.Children.Add(letter);
                CaptchaText = chars;
            }

            for (int i = 0; i < 20; i++)
            {
                Line line = new Line()
                {
                    StartPoint = new Avalonia.Point(rnd.Next(canvaWidth), rnd.Next(canvaHeight)),
                    EndPoint = new Avalonia.Point(rnd.Next(canvaWidth), rnd.Next(canvaHeight)),
                    Stroke = new SolidColorBrush(Color.FromArgb(Convert.ToByte(rnd.Next(150)), Convert.ToByte(rnd.Next(256)), Convert.ToByte(rnd.Next(256)), Convert.ToByte(rnd.Next(255)))),
                    StrokeThickness = 3
                };

                canvas.Children.Add(line);
            }

            Canva = canvas;
            CaptchaVisibility = true;
        }

        private int rotateNum(int num) {
            Random rnd = new Random();
            if (num == 0)
                return rnd.Next(285, 361);
            else
                return rnd.Next(76);
        }

        public bool checkCaptcha() {
            if (string.Equals(CaptchaText, CaptchaEntry) && ModelPassword == Password && ModelEmail == Email) {
                return true;
                
            }
            return false;
        }


    }
}